#!/usr/bin/perl
use strict;

use MIME::Base64;
use File::Basename;
use Net::IMAP::Client;
#use Data::Dumper;
use POSIX qw(ceil);

# Greg Kennedy 2011
#  treat an IMAP mailbox as a file system

# settings here.
#   recommended a multiple of 57 and under 10mb
my $max_attach_size = 5.7 * 1000 * 1024;

my $imap = Net::IMAP::Client->new(
    server => "imap.gmail.com",
    user   => 'username@gmail.com',
    pass   => 'password',
    ssl    => 1,
    port   => 993
  ) or die "Could not connect to IMAP server";

sub usage
{
  print "Usage: mail-drive.pl <command> <options>\n where <command> is one in:\n  get <file>\n  put <file>\n  del <file>\n  list\n  free\n";
}

# Perl trim function to remove whitespace from the start and end of the string
sub trim($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

if (scalar @ARGV == 1 && $ARGV[0] eq 'list'){
  $imap->login or die "Login failed: " . $imap->last_error;
  $imap->select("Inbox");
  my $files = $imap->search( 'ALL' );
# get the subject line from each message
  my $subjects = $imap->fetch($files,"BODY[HEADER.FIELDS (Subject)]");
  foreach my $file(@$subjects)
  {
    my $subj = trim($file->{"BODY[HEADER.FIELDS (Subject)]"});
    my $pos = index($subj,'FILE PART 0001/');

#utf8 subjectline workaround
    if ($pos < 0) { $pos = index($subj,'FILE_PART_0001/'); }

    if ($pos >= 0)
    {
      my @details = split(/\|/, substr($subj,$pos));
      print $details[2] . "\t" . $details[1] . "\n";
    }
  }
  $imap->logout();

} elsif (scalar @ARGV == 1 && $ARGV[0] eq 'free') {
  $imap->login or die "Login failed: " . $imap->last_error;
  $imap->select("Inbox");
  foreach ( @{$imap->capability} ) { print $_ . "\n"; }
print Dumper($imap);
  $imap->logout();

} elsif (scalar @ARGV == 2 && $ARGV[0] eq 'get') {
  $imap->login or die "Login failed: " . $imap->last_error;
  $imap->select("Inbox");
  my $files = $imap->search( 'ALL' );
# get the subject line from each message
  my $subjects = $imap->fetch($files,"BODY[HEADER.FIELDS (Subject)]");
  print "Getting " . $ARGV[1] . "...\n";
  open (FH,">$ARGV[1]") or die "could not open file $ARGV[1]: $!\n";
  foreach my $file(sort(@$subjects))
  {
    my $subj = trim($file->{"BODY[HEADER.FIELDS (Subject)]"});
    if (substr($subj,-1-length($ARGV[1])) eq "|$ARGV[1]")
    {
      print " " . $subj . "\n";
      my $content = $imap->get_part_body($file->{"UID"},'1');
      print FH decode_base64($$content);
    }
  }
  close FH;
  $imap->logout();

} elsif (scalar @ARGV == 2 && $ARGV[0] eq 'put') {
  $imap->login or die "Login failed: " . $imap->last_error;
  open (FH,"$ARGV[1]") or die "could not open file $ARGV[1]: $!\n";
  my $size = -s FH;
  my $part = 1;
  print "Putting " . $ARGV[1] . "...\n";
  while (read(FH,my $block,$max_attach_size))
  {
    my $msg = "From: " . $imap->{user} . "\n";
    $msg   .= "To: " . $imap->{user} . "\n";
    $msg   .= "Subject: FILE PART " . sprintf("%04u",$part) . "/" . sprintf("%04u", ceil($size / $max_attach_size)) . "|" . $size . "|" . basename($ARGV[1]) . "\n\n";
    print " encoding...\n";
    $msg   .= encode_base64($block);
    print "   posting...\n";
 
    $imap->append("Inbox",\$msg);
    print " part " . sprintf("%04u",$part) . "/" . sprintf("%04u", ceil($size / $max_attach_size)) . "\n";
    $part ++;
  }
  close FH;
  $imap->logout();

} elsif (scalar @ARGV == 2 && $ARGV[0] eq 'del') {
  $imap->login or die "Login failed: " . $imap->last_error;
  $imap->select("Inbox");
  my $files = $imap->search( 'ALL' );
# get the subject line from each message
  my $subjects = $imap->fetch($files,"BODY[HEADER.FIELDS (Subject)]");
  foreach my $file(sort(@$subjects))
  {
    my $subj = trim($file->{"BODY[HEADER.FIELDS (Subject)]"});
    if (substr($subj,-1-length($ARGV[1])) eq "|$ARGV[1]")
    {
      print " " . $subj . "\n";
# this is a gmail hack
      $imap->copy($file->{"UID"},"[Gmail]/Trash");
      $imap->delete_message($file->{"UID"});
    }
  }
  $imap->expunge();
  $imap->logout();

} elsif (scalar @ARGV == 1 && $ARGV[0] eq 'free') {
  $imap->login or die "Login failed: " . $imap->last_error;
  $imap->select("Inbox");
  foreach ( @{$imap->capability} ) { print $_ . "\n"; }
} else {
  usage();
  exit;
}
